import logging
import os

from ..filetypes.utils import read_file_to_hex_data_by_path, load_signature_by_hex_data, \
    read_file_to_hex_data_by_file_binary, get_max_offset_extensions

logger = logging.getLogger()


class File(object):

    def __init__(self):
        pass

    @staticmethod
    def validate_input_filepath(filepath):
        if not os.path.isfile(filepath):
            message = "[ERROR] :: filepath %s not exist." % filepath
            logger.error(message)
            raise Exception(message)
        return

    @staticmethod
    def validate_input_extension(extensions):
        if not extensions:
            logger.error("extensions must be exists")
            raise Exception("[ERROR]")
        return

    @staticmethod
    def get_extension_by_filepath(filepath, max_offset_extensions):
        # Validate filepath
        File.validate_input_filepath(filepath)
        hex_data_file = read_file_to_hex_data_by_path(filepath, max_offset_extensions)

        lst_extension = load_signature_by_hex_data(hex_data_file)

        return lst_extension

    @staticmethod
    def get_extension_by_file_binary(file_binary, max_offset_extensions):
        # Validate filepath
        hex_data_file = read_file_to_hex_data_by_file_binary(file_binary, max_offset_extensions)

        lst_extension = load_signature_by_hex_data(hex_data_file)

        return lst_extension

    @staticmethod
    def check_filetype_by_file_extensions(filepath=None, file_binary=None, extensions=None):
        File.validate_input_extension(extensions)

        max_offset_extensions = get_max_offset_extensions(extensions)

        if filepath:
            lst_extension = File.get_extension_by_filepath(filepath, max_offset_extensions)
        else:
            lst_extension = File.get_extension_by_file_binary(file_binary, max_offset_extensions)
        result_merge_extension = list(set(lst_extension) & set(extensions))
        result = {
            "status": len(result_merge_extension) > 0,
            "extension_of_file": lst_extension
        }
        return result



import binascii
import json

from ..filetypes.common import FILEPATH_SIGNATURE, ParamFileSignature


def read_file_to_hex_data_by_path(filepath, max_offset_extensions):
    # Open in binary mode (so you don't read two byte line endings on Windows as one byte)
    # and use with statement (always do this to avoid leaked file descriptors, unflushed files)
    with open(filepath, 'rb') as f:
        # Slurp the whole file and efficiently convert it to hex all at once
        hex_data = binascii.hexlify(f.read(max_offset_extensions))
    return hex_data


def read_file_to_hex_data_by_file_binary(file_binary, max_offset_extensions):
    stream_read = file_binary.stream.read()
    file_binary.seek(0)
    hex_data = binascii.hexlify(stream_read)
    return hex_data


def load_signature_by_hex_data(hex_data_file):
    results = []
    with open(FILEPATH_SIGNATURE, "r") as f:
        data_signature = f.read()
        data_signature = json.loads(data_signature)
        for extension, signature in data_signature.items():
            lst_signs = signature.get(ParamFileSignature.SIGNS)
            for sign_data in lst_signs:
                signs = sign_data.split(",")
                offset, hex_code = int(signs[0]), signs[1]
                len_hex_code = len(hex_code)
                hex_data_slice = hex_data_file[offset: len_hex_code].decode('utf-8')
                if hex_code.lower() == hex_data_slice.lower() and extension not in results:
                    results.append(extension)
    return results


def get_max_offset_extensions(extensions):
    max_offset_extensions = 0
    with open(FILEPATH_SIGNATURE, "r") as f:
        data_signature = f.read()
        data_signature = json.loads(data_signature)
        for extension, signature in data_signature.items():
            if extension not in extensions:
                continue
            lst_signs = signature.get(ParamFileSignature.SIGNS)
            for sign_data in lst_signs:
                signs = sign_data.split(",")
                offset, hex_code = int(signs[0]), signs[1]
                len_hex_code = len(hex_code)

                if len_hex_code > max_offset_extensions:
                    max_offset_extensions = len_hex_code
    return max_offset_extensions




import os
import pathlib

from setuptools import setup, find_packages

with open('README.md', 'r') as fh:
    long_description = fh.read()

__PACKAGE_NAME__ = 'truestrive'
__TEMPLATE_DIR__ = 'libs'


def package_data_files(directory):
    paths = []
    for (path, directories, filenames) in os.walk(directory):
        p = pathlib.Path(path)
        path = pathlib.Path(*p.parts[1:])
        for filename in filenames:
            paths.append(os.path.join(path, filename))
    return paths


data_files = package_data_files(os.path.join(__PACKAGE_NAME__, __TEMPLATE_DIR__))

setup(
    name='validate-extension-file',
    version='0.0.1',
    author='TrueStrive',
    author_email='',
    description='Libs check filetype',
    long_description=long_description,
    long_description_content_type='text/markdown',
    entry_points={
    },
    classifiers=[
        'Programming Language :: Python',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
    ],
    packages=find_packages(),
    package_data={'': data_files}
)



import os

PATH_FOLDER, _ = os.path.split(os.path.abspath(__file__))

FILEPATH_SIGNATURE = os.path.join(PATH_FOLDER, 'file_signature.json')


class ExtensionImage:
    GIF = "gif"
    JPG = "jpg"
    JPEG = "jpeg"
    PNG = "png"

    LIST_EXTENSION_SUPPORTED = [GIF, JPEG, PNG, JPG]


class ExtensionDocument:
    DOC = "doc"
    DOCX = "docx"
    XLS = "xls"
    XLSX = "xlsx"
    PPT = "ppt"
    PPTX = "pptx"
    PDF = "pdf"

    LIST_EXTENSION_SUPPORTED = [DOC, DOCX, XLS, XLSX, PPT, PPTX, PDF]


class ExtensionAudio:
    MP3 = "mp3"
    MP4 = "mp4"
    MOV = "mov"
    AVI = "avi"

    LIST_EXTENSION_SUPPORTED = [MP3, MP4, MOV, AVI]


class ParamFileSignature:
    SIGNS = "signs"
    MIME = "mime"
